namespace $rootnamespace$
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using EnvDTE;

    public partial class $safeitemrootname$
    {
        private Dictionary<string, List<CodeInterface>> codeElements = null;

        /// <summary>
        /// Gets a dictionary containing the interfaces defined in the code generation project item.
        /// The dictionary is keyed on the interface's full name. The values are lists containing the partial
        /// definitions for that interface.
        /// </summary>
        public Dictionary<string, List<CodeInterface>> CodeElements
        {
            get
            {
                if (this.codeElements == null)
                {
                    // the interfaces defined in this file that reference at least one data interface.
                    this.codeElements = base.ProjectItem.GetCodeElements<CodeInterface>();
                }

                return this.codeElements;
            }
        }

        private void RenderMethods(EnvDTE.CodeInterface interf)
        {
            (from i in this.CodeElements[interf.FullName]                       // for all partial definitions of the interface
             from codeElement in i.Members.Cast<CodeElement>()                   // for all code elements in the partial definitions
             where codeElement.Kind == EnvDTE.vsCMElement.vsCMElementFunction    // that are methods
             select codeElement).Cast<EnvDTE.CodeFunction>().ToList()            // select them as a list of CodeFunction
             .ForEach(codeFunction => 
             {
                 RenderMethod(codeFunction);
                 this.Output.WriteLine();
             });
        }

        /// <summary>
        /// Renders a method parameters as a list.
        /// e.g. <c>string fn(string s1, int i1)</c> will result in <c>s1, i1</c>.
        /// </summary>
        /// <param name="method">The method.</param>
        /// <returns>A string containing the paremeters' names separated by commas.</returns>
        public static string MethodParametersDeclaration(EnvDTE.CodeFunction method)
        {
            return Join(method.Parameters, ", ", (EnvDTE80.CodeParameter2 parameter) =>
                {
                    string prefix = string.Empty;
                    if ((parameter.ParameterKind & EnvDTE80.vsCMParameterKind.vsCMParameterKindRef) != 0)
                    {
                        prefix = "ref";
                    }

                    if ((parameter.ParameterKind & EnvDTE80.vsCMParameterKind.vsCMParameterKindParamArray) != 0)
                    {
                        prefix += " params";
                    }

                    return string.Format("{0}{1} {2}", prefix, parameter.Type.AsString, parameter.Name);
                });
        }
    }
}